/* FeatureEdit.java
 *
 * created: Tue Dec  1 1998
 *
 * This file is part of Artemis
 *
 * Copyright (C) 1998,1999  Genome Research Limited
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/diana/components/FeatureEdit.java,v 1.34 1999/09/14 10:11:42 kmr Exp $
 **/

package diana.components;

import Diana;
import uk.ac.sanger.pathogens.*;
import diana.*;
import diana.sequence.*;
import uk.ac.sanger.pathogens.embl.OutOfDateException;
import uk.ac.sanger.pathogens.embl.LocationParseException;
import uk.ac.sanger.pathogens.embl.InvalidRelationException;
import uk.ac.sanger.pathogens.embl.QualifierParseException;
import uk.ac.sanger.pathogens.embl.Key;
import uk.ac.sanger.pathogens.embl.Location;
import uk.ac.sanger.pathogens.embl.Qualifier;
import uk.ac.sanger.pathogens.embl.EntryInformation;
import uk.ac.sanger.pathogens.embl.StreamQualifier;
import uk.ac.sanger.pathogens.embl.QualifierInfo;
import uk.ac.sanger.pathogens.embl.EmblStreamFeature;

import java.awt.*;
import java.awt.event.*;
import java.io.StringReader;
import java.io.IOException;
import java.util.Date;

/**
 *  FeatureEdit class
 *
 *  @author Kim Rutherford
 *  @version $Id: FeatureEdit.java,v 1.34 1999/09/14 10:11:42 kmr Exp $
 **/

public class FeatureEdit
    extends Frame
    implements EntryChangeListener, FeatureChangeListener {
  /**
   *  Create a new FeatureEdit object from the given Feature.
   **/
  public FeatureEdit (Feature edit_feature) {
    super ("Artemis Feature Edit: " + edit_feature.getIDString ());

    this.edit_feature = edit_feature;
    this.edit_entry = edit_feature.getEntry ();

    final Font font = Options.getOptions ().getFont ();

    setFont (font);

    createComponents ();

    updateFromFeature ();

    edit_feature.getEntry ().addEntryChangeListener (this);
    edit_feature.addFeatureChangeListener (this);

    addWindowListener (new WindowAdapter () {
      public void windowClosing (WindowEvent event) {
        stopListening ();
        dispose ();
      }
    });

    pack ();

    final Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    setLocation (new Point ((screen.width - getSize ().width) / 2,
                            (screen.height - getSize ().height) / 2));
  }

  /**
   *  Remove this object as a feature and entry change listener.
   **/
  public void stopListening () {
    getEntry ().removeEntryChangeListener (this);
    getFeature ().removeFeatureChangeListener (this);
  }

  /**
   *  Implementation of the EntryChangeListener interface.  We listen to
   *  EntryChange events so we can notify the user if of this component if the
   *  feature gets deleted.
   **/
  public void entryChanged (EntryChangeEvent event) {
//    System.out.println ("FeatureEdit.entryChanged ()");
    switch (event.getType ()) {
    case EntryChangeEvent.FEATURE_DELETED:
      if (event.getFeature () == edit_feature) {
        stopListening ();
        dispose ();
      }
      break;
    default:
      // do nothing;
      break;
    }
  }

  /**
   *  Implementation of the FeatureChangeListener interface.  We need to
   *  listen to feature change events from the Features in this object so that
   *  we can update the display.
   *  @param event The change event.
   **/
  public void featureChanged (FeatureChangeEvent event) {

//      System.out.println ("FeatureEdit: feature change event type: " +
//                          event.getType () + " source " +
//                          event.getSource ());

    // re-read the information from the feature
    switch (event.getType ()) {
    case FeatureChangeEvent.LOCATION_CHANGED:
      updateLocation ();
      break;
    case FeatureChangeEvent.KEY_CHANGED:
      updateKey ();
      break;
    default:
      updateFromFeature ();
      break;
    }
  }


  /**
   *  Create all the components for this FeatureEdit component.
   **/
  private void createComponents () {
    key_choice = new KeyChoice (getFeature ().getKey ());

    final Panel key_and_qualifier_panel = new Panel ();

    key_and_qualifier_panel.setFont (getFont ());
    location_text.setFont (getFont ());
    add_qualifier_button.setFont (getFont ());
    ok_button.setFont (getFont ());
    cancel_button.setFont (getFont ());
    update_button.setFont (getFont ());

    key_and_qualifier_panel.setLayout (new BorderLayout ());
    key_and_qualifier_panel.add (key_choice, "West");

    qualifier_choice = new QualifierChoice (key_choice.getSelectedItem ());

    final Panel qualifier_panel = new Panel ();

    qualifier_panel.add (new Label ("Add Qualifier: "));
    qualifier_panel.add (qualifier_choice);

    key_and_qualifier_panel.add (qualifier_panel, "East");

    key_choice.addItemListener (new ItemListener () {
      public void itemStateChanged (ItemEvent _) {
        qualifier_choice.setKey (key_choice.getSelectedItem ());
      }
    });

    qualifier_choice.addItemListener (new ItemListener () {
      public void itemStateChanged (ItemEvent _) {
        final String qualifier_name = qualifier_choice.getSelectedItem ();
        final QualifierInfo qualifier_info =
          EntryInformation.getQualifierInfo (qualifier_name);

        if (qualifier_info == null) {
          new MessageDialog (FeatureEdit.this, "internal error: no " +
                             "qualifier info for " + qualifier_name);
        } else {
          qualifier_text_area.append ("/" + qualifier_name);

          switch (qualifier_info.getType ()) {
          case QualifierInfo.QUOTED_TEXT:
            qualifier_text_area.append ("=\"\"");
            break;

          case QualifierInfo.NO_VALUE:
          case QualifierInfo.OPTIONAL:
            break;

          default:
            qualifier_text_area.append ("=");
          }

          qualifier_text_area.append ("\n");
        }
      }
    });

    final Panel middle_panel = new Panel ();

    middle_panel.setLayout (new BorderLayout ());

    final Panel location_panel = new Panel ();
    location_panel.add (new Label ("location: "));
    location_panel.add (location_text);

    middle_panel.add (location_panel, "North");

    add (key_and_qualifier_panel, "North");

    cancel_button.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        stopListening ();
        dispose ();
      }
    });

    ok_button.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        if (setFeature ()) {
          stopListening ();
          dispose ();
        }
      }
    });

    update_button.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        setFeature ();
      }
    });

    final FlowLayout flow_layout =
      new FlowLayout (FlowLayout.CENTER, 18, 5);

    final Panel ok_cancel_update_panel = new Panel (flow_layout);

    ok_cancel_update_panel.setFont (getFont ());

    ok_cancel_update_panel.add (ok_button);
    ok_cancel_update_panel.add (cancel_button);
    ok_cancel_update_panel.add (update_button);

    add (ok_cancel_update_panel, "South");


    middle_panel.add (qualifier_text_area, "Center");
    qualifier_text_area.setFont (getFont ());

    add (middle_panel, "Center");

//      final Panel qualifier_panel = new Panel ();

//      GridBagLayout gridbag = new GridBagLayout();
//      qualifier_panel.setLayout (gridbag);

//      GridBagConstraints c = new GridBagConstraints();

//      c.gridwidth = GridBagConstraints.REMAINDER;
//      c.fill = GridBagConstraints.HORIZONTAL;
//      c.anchor = GridBagConstraints.NORTH;
//      c.gridheight = 1;
//      c.weightx = 1;
//      c.weighty = 0;

//      // contains just the add button
//      final Panel add_panel = new Panel ();

//      add_panel.add (add_qualifier_button);

//      gridbag.setConstraints (add_panel, c);
//      qualifier_panel.add (add_panel);
  }

  /**
   *  Read the key, location and qualifier information from the feature and
   *  update the components.
   **/
  private void updateFromFeature () {
    datestamp = getFeature ().getDatestamp ();

    updateKey ();

    updateLocation ();

    updateQualifiers ();
  }

  /**
   *  Read the location from the feature and update the location field.
   **/
  private void updateLocation () {
    location_text.setText (getFeature ().getLocation ().toStringJoinInner ());
  }

  /**
   *  Read the qualifiers from the feature and update the qualifier TextArea.
   **/
  private void updateQualifiers () {


    qualifier_text_area.setText ("");

    final QualifierVector qualifiers = getFeature ().getQualifiers ();

    for (int i = 0 ; i < qualifiers.size () ; ++i) {

      final StringVector qualifier_strings =
        StreamQualifier.toStringVector (qualifiers.elementAt (i));

      for (int value_index = 0 ;
           value_index < qualifier_strings.size () ;
           ++value_index) {

        final String qualifier_string =
          qualifier_strings.elementAt (value_index);

        qualifier_text_area.append (qualifier_string + "\n");
      }
    }
  }

  /**
   *  Set the key, location and qualifiers of the feature to be the same as
   *  what values currently shown in the components.
   *  @return true if and only if action succeeds.  It may fail because of an
   *    illegal location or qualifier, in which case a message will be
   *    displayed before returning.
   **/
  private boolean setFeature () {
    final Key key = key_choice.getSelectedItem ();

    if (! EntryInformation.getPossibleKeys ().contains (key.toString ())) {
      final YesNoDialog dialog =
        new YesNoDialog (this, "Add this new key: " + key + "?");

      if (dialog.getResult ()) {
        // yes
        EntryInformation.addPossibleKey (key.toString ());
      } else {
        // no
        return false;
      }
    }

    final Location location;

    try {
      location = new Location (location_text.getText ());
    } catch (LocationParseException exception) {
      final String error_string = exception.getMessage ();
      System.out.println (error_string);
      new MessageDialog (this,
                         "Cannot apply changes because of location error: " +
                         error_string);

      return false;
    }


    final QualifierVector qualifiers;

    try {
      final String qualifier_string = qualifier_text_area.getText ();
      qualifiers = getQualifiersFromString (qualifier_string);
    } catch (QualifierParseException exception) {
      final String error_string = exception.getMessage ();
      System.out.println (error_string);
      new MessageDialog (this,
                         "Cannot apply changes because of a qualifier " +
                         "error: " + error_string);

      return false;
    }


    try {
      try {
        getFeature ().set (datestamp, key, location, qualifiers);
      } catch (OutOfDateException e) {
        final YesNoDialog dialog =
          new YesNoDialog (this,
                           "the feature has changed since the edit " +
                           "window was opened, continue?");
        if (dialog.getResult ()) {
          // yes - ignore the datestamp
          getFeature ().set (key, location, qualifiers);
        } else {
          // no
          return false;
        }
      }
    } catch (InvalidRelationException exception) {
      final String error_string = exception.getMessage ();
      System.out.println (error_string);
      new MessageDialog (this, "Cannot apply changes: " + error_string);

      return false;
    } catch (OutOfRangeException e) {
      new MessageDialog (this,
                         "Cannot apply changes - the location is out of " +
                         "range for this sequence");
      return false;
    } catch (ReadOnlyException e) {
      new MessageDialog (this,
                         "Cannot apply changes - the feature is in a " +
                         "read only entry");
      return false;
    }

    return true;
  }

  /**
   *  Return the Feature we are editing.
   **/
  public Feature getFeature () {
    return edit_feature;
  }

  /**
   *  Return a QualifierVector containing the qualifiers from a String.
   *  @param qual_string contains the qualifiers to parse
   */
  private static QualifierVector getQualifiersFromString (String qual_string)
      throws QualifierParseException {

    final StringReader string_reader = new StringReader (qual_string);

    final QualifierVector qualifiers = new QualifierVector ();

//    System.out.println (qual_string);

    try {
      final uk.ac.sanger.pathogens.embl.QualifierVector embl_qualifiers =
        EmblStreamFeature.readQualifiers (string_reader);

      for (int i = 0 ; i < embl_qualifiers.size () ; ++i) {
        qualifiers.add (embl_qualifiers.elementAt (i));
      }
    } catch (IOException exception) {
      throw (new QualifierParseException (exception.getMessage ()));
    }

    string_reader.close ();

    return qualifiers;
  }

  /**
   *  Return the Entry that contains the Feature this object is displaying.
   **/
  private Entry getEntry () {
    return edit_entry;
  }

  /**
   *  Read the key from the feature and update the key chooser.
   **/
  private void updateKey () {
    final Key feature_key = getFeature ().getKey ();

    key_choice.setKey (feature_key);
  }


  /**
   *  The choice of feature keys.
   **/
  private KeyChoice key_choice;

  private QualifierChoice qualifier_choice = null;

  private TextField location_text = new TextField (80);

  private Button add_qualifier_button = new Button ("Add qualifier");

  private Button ok_button = new Button ("OK");
  private Button cancel_button = new Button ("Cancel");
  private Button update_button = new Button ("Apply");

  private TextArea qualifier_text_area =
    new TextArea ("", 18, 81, TextArea.SCROLLBARS_VERTICAL_ONLY);

  /**
   *  The Feature this object is displaying.
   **/
  private Feature edit_feature;

  /**
   *  The Entry that contains the Feature this object is displaying.
   **/
  private Entry edit_entry;

  /**
   *  The datestamp of the RWCorbaFeature when updateFromFeature () was last
   *  called or null if this is not a RWCorbaFeature.
   **/
  private Date datestamp = null;

  /**
   *  A Label for showing errors and messages.
   **/
  private Label error_text = new Label ("");
}
