/* DocumentEntryFactory.java
 *
 * created: Sat Sep 11 1999
 *
 * This file is part of Artemis
 * 
 * Copyright (C) 1999  Genome Research Limited
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/uk/ac/sanger/pathogens/embl/DocumentEntryFactory.java,v 1.4 1999/10/29 14:57:29 kmr Exp $
 */

package uk.ac.sanger.pathogens.embl;

import uk.ac.sanger.pathogens.*;

import java.io.*;

/**
 *  This class contains the method makeDocumentEntry (), which creates a
 *  DocumentEntry object of the appropriate class from a Document object.
 *
 *  @author Kim Rutherford
 *  @version $Id: DocumentEntryFactory.java,v 1.4 1999/10/29 14:57:29 kmr Exp $
 **/

abstract public class DocumentEntryFactory {
  /**
   *  Read a DocumentEntry object from the given Document.
   **/
  public static DocumentEntry makeDocumentEntry (final Document document)
      throws IOException, InvalidRelationException {

    final Reader document_reader = document.getReader ();

    final BufferedReader buffered_reader =
      new BufferedReader (document_reader);

    final String first_line = buffered_reader.readLine ();

    if (first_line == null) {
      // empty file - create an empty EmblDocumentEntry
      return new EmblDocumentEntry (document);
    }

    final int first_line_type = LineGroup.getLineType (first_line);

    if (first_line_type == LineGroup.GFF_LINE) {
      return new GFFDocumentEntry (document);
    }

    if (first_line_type == LineGroup.GENBANK_MISC ||
        first_line_type == LineGroup.GENBANK_FEATURE_TABLE) {
      return new GenbankDocumentEntry (document);
    } else {
      return new EmblDocumentEntry (document);
    }
  }

  /**
   *  Make a new (nameless) DocumentEntry from the given entry.
   *  @param destination_type This parameter control the type of DocumentEntry
   *    that is created.  It should be a DocumentEntry type that can be
   *    constructed from any Entry eg. one of EMBL_FORMAT, GENBANK_FORMAT.
   **/
  public static DocumentEntry makeDocumentEntry (final Entry entry,
                                                 final int destination_type) {
    switch (destination_type) {
    case EMBL_FORMAT:
      return new EmblDocumentEntry (entry);
    case GENBANK_FORMAT:
      return new GenbankDocumentEntry (entry);
    default:
      throw new Error ("internal error - unknown DocumentEntry type");
    }
  }

  /**
   *  The tag use if the fromat of the entry is unknown.
   **/
  final public static int UNKNOWN_FORMAT = 0;

  /**
   *  The tag use if the fromat of the entry is not important.
   **/
  final public static int ANY_FORMAT = 0;

  /**
   *  The tag use for an entry that is in EMBL format.
   **/
  final public static int EMBL_FORMAT = 1;

  /**
   *  The tag use for an entry that is in GENBANK format.
   **/
  final public static int GENBANK_FORMAT = 2;

  /**
   *  The tag use for an entry that is in GFF format.
   **/
  final public static int GFF_FORMAT = 2;
}


