/* QualifierInfo.java
 *
 * created: Sun Feb 21 1999
 *
 * This file is part of Artemis
 * 
 * Copyright (C) 1998,1999  Genome Research Limited
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/uk/ac/sanger/pathogens/embl/QualifierInfo.java,v 1.2 1999/09/07 16:21:55 kmr Exp $
 */

package uk.ac.sanger.pathogens.embl;

/**
 *  Each object of this class contains the information about one qualifier
 *  type.
 *
 *  @author Kim Rutherford
 *  @version $Id: QualifierInfo.java,v 1.2 1999/09/07 16:21:55 kmr Exp $
 **/

public class QualifierInfo {
  /**
   *  Create a new QualifierInfo object.
   *  @exception QualifierInfoException Thrown if the type_string doesn't
   *    contain a valid type string.
   **/
  public QualifierInfo (final String name, final String type_string)
      throws QualifierInfoException {
    // now tidy up the names - remove the * or @ character (if any) from the
    // start of each name
    if (name.startsWith ("*") || name.startsWith ("@")) {
      this.name = name.substring (1);
    } else {
      this.name = name;
    }

    this.type_string = type_string;

    type = getTypeID (type_string);

    if (type == UNKNOWN) {
      throw new QualifierInfoException ("unknown qualifier type: " +
                                        type_string);
    }
  }

  /**
   *  Returned by getType () when the value of the qualifier should a text
   *  string quoted with double quotes: "some text".
   **/
  public final static int QUOTED_TEXT = 1;

  /**
   *  Returned by getType () when the value of the qualifier should be an
   *  unquoted text string (containing no spaces).
   **/
  public final static int TEXT        = 2;

  /**
   *  Returned by getType () if this qualifier not take a value (for example
   *  /partial)
   **/
  public final static int NO_VALUE    = 3;

  /**
   *  Returned by getType () if the value of this qualifier is optional
   **/
  public final static int OPTIONAL    = 4;

  /**
   *  Returned getType () if
   **/
  private final static int UNKNOWN    = 0;

  /**
   *  Return\s the type ID of this QualifierInfo object.
   **/
  public int getType () {
    return type;
  }

  /**
   *  Return the name of the QualifierInfo object ("label", "note", etc.)
   **/
  public String getName () {
    return name;
  }
  
  /**
   *  A String array containing the possible type strings.  This is used by
   *  getTypeID () to check that a valid type string was passed to the
   *  constructor.
   **/
  private static final String [] possible_type_strings = {
    "\"text\"", "text", "\"list\"", "\"opt\"", "(::)", "EC", "feature", "list",
    "location", "modbase", "none", "number", "real", "ref", "item"
  };

  /**
   *  Return the type ID for the given type_string.
   **/
  private static int getTypeID (final String type_string) {
    for (int i = 0 ; i < possible_type_strings.length ; ++i) {
      final String this_type_string = possible_type_strings[i];

      if (this_type_string.equals (type_string)) {

        // found a match so return the correct ID

        if (type_string.equals ("\"text\"") ||
            type_string.equals ("\"list\"")) {
          return QUOTED_TEXT;
        } else {
          if (type_string.equals ("text")) {
            return TEXT;
          } else {
            if (type_string.equals ("none")) {
              return NO_VALUE;
            } else {
              if (type_string.equals ("\"opt\"")) {
                return OPTIONAL;
              } else {
                // default is text which covers all the non quoted type
                return TEXT;
              }
            }
          }
        }
      }
    }

    return UNKNOWN;
  }

  /**
   *  The name string that was passed to the constructor.
   **/
  private final String name;

  /**
   *  The type string that was passed to the constructor.
   **/
  private final String type_string;

  /**
   *  The type ID of this QualifierInfo object (set in the constructor using
   *  getTypeID ()
   **/
  private final int type;
}


